@@ -41,23 +41,43 @@
             man_content_tree/2,         % +Dir, -Tree
             man_packages_tree/1         % -Tree
           ]).
-:- use_module(library(lists)).
-:- use_module(library(uri)).
-:- use_module(library(apply)).
-:- use_module(library(option)).
-:- use_module(library(filesex)).
-:- use_module(library(xpath)).
-:- use_module(doc_html).
-:- use_module(doc_search).
-:- use_module(doc_process).
-:- use_module(doc_util).
-:- use_module(man_index).
+:- use_module(library(xpath),[xpath/3, op(_,_,_)]).
 :- use_module(library(http/html_write)).
-:- use_module(library(http/html_head)).
-:- use_module(library(http/http_dispatch)).
-:- use_module(library(http/http_path)).
-:- use_module(library(http/mimetype)).
-:- use_module(library(prolog_xref)).
+
+:- autoload(doc_html,
+	    [ object_tree/5, private/2, object_page_header/4, objects/4,
+	      object_href/2, object_synopsis/4, object_page_footer/4,
+	      object_ref/4, object_page/4,
+              object_source_button//2
+	    ]).
+:- autoload(doc_process,[doc_comment/4]).
+:- autoload(doc_search,[search_form/3]).
+:- autoload(doc_util,[atom_to_object/2,atom_pi/2]).
+:- autoload(man_index,[manual_object/5]).
+:- autoload(library(apply),[maplist/2,maplist/3]).
+:- autoload(library(debug),[assertion/1,debug/3]).
+:- autoload(library(error),[permission_error/3,existence_error/2]).
+:- autoload(library(filesex),
+	    [directory_file_path/3,relative_file_name/3]).
+:- autoload(library(lists),
+	    [select/4,append/3,member/2,last/2,selectchk/3]).
+:- autoload(library(option),[merge_options/3,option/2,option/3]).
+:- autoload(library(pairs),[pairs_values/2,pairs_keys/2]).
+:- autoload(library(prolog_xref),[xref_public_list/3]).
+:- autoload(library(sgml),
+	    [ load_html/3, dtd/2, new_sgml_parser/2, set_sgml_parser/2,
+	      sgml_parse/2, free_sgml_parser/1
+	    ]).
+:- autoload(library(uri),[uri_encoded/3]).
+:- autoload(library(www_browser),[expand_url_path/2]).
+:- autoload(library(http/html_head),[html_requires/3]).
+:- autoload(library(http/http_dispatch),
+	    [ http_link_to_id/3, http_location_by_id/2,
+	      http_handler/3, http_reply_file/3, http_redirect/3
+	    ]).
+:- autoload(library(http/http_path),[http_absolute_location/3]).
+:- autoload(library(http/mimetype),[file_mime_type/2]).
+
 :- include(hooks).
 
 /** <module> Process SWI-Prolog HTML manuals
@@ -726,7 +746,7 @@ man_match(packages, packages, _) -->
 
           <p>
           See also <a href="/pack/list">Add-ons</a> for extensions
-          provided by the community that must be installed seperately
+          provided by the community that must be installed separately
           using
           <a href="/pldoc/doc_for?object=pack_install/1">pack_install/1</a>.</p>
          |}).
@@ -746,41 +766,41 @@ man_match((Parent+Path)-(Obj+[element(dt,A,C0)|DD]), Obj, Options) -->
     dom_list([ element(dt,[],[\man_synopsis(QObj, Section, LibOpt)]),
                element(dt,A,C)
              | DD
-             ], Path).
-man_match((_Parent+Path)-(Obj+DOM), Obj, _) -->
-    dom_list(DOM, Path).
+             ], Path, Options).
+man_match((_Parent+Path)-(Obj+DOM), Obj, Options) -->
+    dom_list(DOM, Path, Options).
 
 
 :- html_meta
-    dom_list(html, +, ?, ?).
+    dom_list(html, +, +, ?, ?).
 
-dom_list(_:[], _) -->
+dom_list(_:[], _, _) -->
     !,
     [].
-dom_list(M:[H|T], Path) -->
-    dom(H, Path),
-    dom_list(M:T, Path).
+dom_list(M:[H|T], Path, Options) -->
+    dom(H, Path, Options),
+    dom_list(M:T, Path, Options).
 
-dom(element(E, Atts, Content), Path) -->
+dom(element(E, Atts, Content), Path, Options) -->
     !,
-    dom_element(E, Atts, Content, Path).
-dom(CDATA, _) -->
+    dom_element(E, Atts, Content, Path, Options).
+dom(CDATA, _, _) -->
     html(CDATA).
 
-dom_element(a, _, [], _) -->                   % Useless back-references
+dom_element(a, _, [], _, _) -->                % Useless back-references
     !,
     [].
-dom_element(a, Att, Content, Path) -->
+dom_element(a, Att, Content, Path, Options) -->
     { memberchk(href=HREF, Att),
       (   memberchk(class=Class, Att)
       ->  true
       ;   Class = unknown
       ),
-      rewrite_ref(Class, HREF, Path, Myref)
+      rewrite_ref(Class, HREF, Path, Myref, Options)
     },
     !,
-    html(a(href(Myref), \dom_list(Content, Path))).
-dom_element(span, Att, [CDATA], _) -->
+    html(a(href(Myref), \dom_list(Content, Path, Options))).
+dom_element(span, Att, [CDATA], _, _Options) -->
     { memberchk(class='pred-ext', Att),
       atom_pi(CDATA, PI),
       documented(PI),
@@ -788,7 +808,7 @@ dom_element(span, Att, [CDATA], _) -->
     },
     !,
     html(a(href(HREF), CDATA)).
-dom_element(img, Att0, [], Path) -->
+dom_element(img, Att0, [], Path, _Options) -->
     { selectchk(src=Src, Att0, Att1),
       relative_file_name(ImgFile, Path, Src),
       handler_alias(Handler, DirAlias),
@@ -806,24 +826,24 @@ dom_element(img, Att0, [], Path) -->
     },
     html_begin(Begin),
     html_end(img).
-dom_element(div, Att, _, _) -->
+dom_element(div, Att, _, _, _) -->
     { memberchk(class=navigate, Att) },
     !.
-dom_element(html, _, Content, Path) -->        % do not emit a html for the second time
-    !,
-    dom_list(Content, Path).
-dom_element(head, _, Content, Path) -->        % do not emit a head for the second time
-    !,
-    dom_list(Content, Path).
-dom_element(title, _, _, _) --> !.
-dom_element(link, _, _, _) --> !.
-dom_element(body, _, Content, Path) -->        % do not emit a body for the second time
-    !,
-    dom_list(Content, Path).
-dom_element(Name, Attrs, Content, Path) -->
+dom_element(html, _, Content, Path, Options) -->
+    !,                              % do not emit a html for the second time
+    dom_list(Content, Path, Options).
+dom_element(head, _, Content, Path, Options) -->
+    !,                              % do not emit a head for the second time
+    dom_list(Content, Path, Options).
+dom_element(title, _, _, _, _) --> !.
+dom_element(link, _, _, _, _) --> !.
+dom_element(body, _, Content, Path, Options) -->
+    !,                              % do not emit a body for the second time
+    dom_list(Content, Path, Options).
+dom_element(Name, Attrs, Content, Path, Options) -->
     { Begin =.. [Name|Attrs] },
     html_begin(Begin),
-    dom_list(Content, Path),
+    dom_list(Content, Path, Options),
     html_end(Name).
 
 handler_alias(manual_file,   swi_man_manual(.)).
@@ -845,7 +865,7 @@ documented(PI) :-
 documented(PI) :-
     full_object(PI, _Obj).
 
-%!  rewrite_ref(+Class, +Ref0, +Path, -ManRef) is semidet.
+%!  rewrite_ref(+Class, +Ref0, +Path, -ManRef, +Options) is semidet.
 %
 %   Rewrite Ref0 from the HTML reference manual format to the server
 %   format. Reformatted:
@@ -860,19 +880,29 @@ documented(PI) :-
 %       $ File#flag:Name :
 %       Rewrite to =|section(Level, NT, ID, FilePath)#flag:Name|=
 %
+%       $ File#gloss:Name :
+%       Rewrite to =|section(Level, NT, ID, FilePath)#gloss:Name|=
+%
 %       $ File#Name()
 %       Rewrite to /man/CAPI=Name
 %
 %   @param Class    Class of the <A>.  Supported classes are
 %
-%           | sec  | Link to a section     |
-%           | pred | Link to a predicate   |
-%           | flag | link to a Prolog flag |
+%           | sec   | Link to a section     |
+%           | pred  | Link to a predicate   |
+%           | flag  | Link to a Prolog flag |
+%           | gloss | Link to a glossary    |
 %
 %   @param Ref0     Initial reference from the =a= element
 %   @param Path     Currently loaded file
 %   @param ManRef   PlDoc server reference
 
+rewrite_ref(_Class, Ref, _Path, Ref, Options) :-
+    option(server(false), Options),
+    !.
+rewrite_ref(Class, Ref0, Path, ManRef, _Options) :-
+    rewrite_ref(Class, Ref0, Path, ManRef).
+
 rewrite_ref(pred, Ref0, _, Ref) :-              % Predicate/DCG reference
     sub_atom(Ref0, _, _, A, '#'),
     !,
@@ -937,6 +967,18 @@ rewrite_ref(flag, Ref0, Path, Ref) :-
     !,
     object_href(Obj, Ref1),
     format(string(Ref), '~w#~w', [Ref1, Fragment]).
+rewrite_ref(gloss, Ref0, Path, Ref) :-
+    sub_atom(Ref0, B, _, A, '#'),
+    !,
+    sub_atom(Ref0, 0, B, _, File),
+    sub_atom(Ref0, _, A, 0, Fragment),
+    file_directory_name(Path, Dir),
+    atomic_list_concat([Dir, /, File], SecPath),
+    Obj = section(_, _, _, SecPath),
+    manual_object(Obj, _, _, _, _),
+    !,
+    object_href(Obj, Ref1),
+    format(string(Ref), '~w#~w', [Ref1, Fragment]).
 
 %!  referenced_section(+Fragment, +File, +Path, -Section)
 
