@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2006-2018, University of Amsterdam
+    Copyright (c)  2006-2019, University of Amsterdam
                               VU University Amsterdam
                               CWI, Amsterdam
     All rights reserved.
@@ -148,11 +148,11 @@ take_block([N-['|'|RL1]|LT], _, Table, Rest) :-
     rest_table(LT, N, RL, Rest),
     !,
     Table = table(class=wiki, [tr(R0)|RL]).
-take_block([0-[-,-|More]|LT], _, Block, LT) :-  % seperation line
+take_block([0-[-,-|More]|LT], _, Block, LT) :-  % separation line
     maplist(=(-), More),
     !,
     Block = hr([]).
-take_block([_-Line|LT], _, Block, LT) :-        % seperation line
+take_block([_-Line|LT], _, Block, LT) :-        % separation line
     ruler(Line),
     !,
     Block = hr([]).
@@ -171,12 +171,26 @@ take_block([I-L1|LT], BaseIndent, Elem, Rest) :-
     rest_par(LT, PT, I, BaseIndent, MaxI, Rest),
     (   MaxI >= BaseIndent+16
     ->  Elem = center(Par)
-    ;   MaxI >= BaseIndent+4
-    ->  Elem = blockquote(Par)
+    ;   phrase(blockquote(BQ), Par)
+    ->  Elem = blockquote(BQ)
     ;   Elem = p(Par)
     ).
 take_block([Verb|Lines], _, Verb, Lines).
 
+blockquote(Clean) -->
+    [>, ' '],
+    bq_lines(Clean).
+
+bq_lines([' '|Par]) -->
+    ['\n'], !, [>,' '],
+    bq_lines(Par).
+bq_lines([H|T]) -->
+    [H],
+    bq_lines(T).
+bq_lines([]) -->
+    [].
+
+
 %!  ruler(+Line) is semidet.
 %
 %   True if Line contains 3 ruler chars and otherwise spaces.
@@ -911,12 +925,12 @@ wiki_face(Code, _, _) -->
           code_face(Text, Term, Vars, Code)
         }
     ).
-wiki_face(Face, _, _) -->
+wiki_face(Face, _, Options) -->
     [ w(Name) ], arg_list(List),
     { atomic_list_concat([Name|List], Text),
       E = error(_,_),
       catch(atom_to_term(Text, Term, Vars), E, fail),
-      term_face(Text, Term, Vars, Face)
+      term_face(Text, Term, Vars, Face, Options)
     },
     !.
 wiki_face(br([]), _, _) -->
@@ -1194,21 +1208,21 @@ arg_list_close([H|T], Depth) -->
     arg_list_close(T, Depth).
 
 
-%!  term_face(+Text, +Term, +Vars, -Face) is semidet.
+%!  term_face(+Text, +Term, +Vars, -Face, +Options) is semidet.
 %
 %   Process embedded Prolog-terms. Currently   processes  Alias(Arg)
 %   terms that refer to files.  Future   versions  will also provide
 %   pretty-printing of Prolog terms.
 
-term_face(_Text, Term, _Vars, \file(Name, FileOptions)) :-
+term_face(_Text, Term, _Vars, \file(Name, FileOptions), Options) :-
     ground(Term),
     compound(Term),
-    compound_name_arguments(Term, Alias, [_]),
+    compound_name_arity(Term, Alias, 1),
     user:file_search_path(Alias, _),
-    existing_file(Term, FileOptions, []),
+    existing_file(Term, FileOptions, [], Options),
     !,
     format(atom(Name), '~q', [Term]).
-term_face(Text, Term, Vars, Face) :-
+term_face(Text, Term, Vars, Face, _Options) :-
     code_face(Text, Term, Vars, Face).
 
 untag([], []).
@@ -1323,6 +1337,15 @@ wiki_link(a(href(Ref), Label), Options) -->
       catch(expand_url_path(Term, Ref), E, fail),
       option(label(Label), Options, Ref)
     }.
+wiki_link(a(href(Ref), Label), Options) -->
+    [#, w(First)],
+    { option(end(End), Options) },
+    tokens_no_whitespace(Rest),
+    peek_end_url(End),
+    !,
+    { atomic_list_concat([#,First|Rest], Ref),
+      option(label(Label), Options, Ref)
+    }.
 wiki_link(a(href(Ref), Label), Options) -->
     [<],
     (   { option(relative(true), Options),
@@ -1357,6 +1380,24 @@ file_name(FileBase, Extension) -->
     ['.'], file_extension(Extension),
     !,
     { atomic_list_concat([S1|List], '/', FileBase) }.
+file_name(FileBase, Extension) -->
+    [w(Alias), '('],
+    { once(user:file_search_path(Alias, _)) },
+    segment(S1),
+    segments(List),
+    [')'],
+    !,
+    { atomic_list_concat([S1|List], '/', Base),
+      Spec =.. [Alias,Base],
+      absolute_file_name(Spec, Path,
+                         [ access(read),
+                           extensions([pl]),
+                           file_type(prolog),
+                           file_errors(fail)
+                         ]),
+      file_name_extension(FileBase, Extension, Path)
+    }.
+
 
 segment(..) -->
     ['.','.'],
@@ -1381,37 +1422,40 @@ file_extension(Ext) -->
     }.
 
 
-%!  resolve_file(+Name, -Options, ?RestOptions) is det.
+%!  resolve_file(+Name, -FileOptions, ?RestOptions, +Options) is det.
 %
 %   Find the actual file based on the pldoc_file global variable. If
 %   present  and  the   file   is    resolvable,   add   an   option
 %   absolute_path(Path) that reflects the current   location  of the
 %   file.
 
-resolve_file(Name, Options, Rest) :-
-    existing_file(Name, Options, Rest),
+resolve_file(Name, FileOptions, Rest) :-
+    existing_file(Name, FileOptions, Rest, []),
     !.
 resolve_file(_, Options, Options).
 
 
-existing_file(Name, Options, Rest) :-
+existing_file(Name, FileOptions, Rest, Options) :-
+    \+ Options.get(link) == false,
     E = error(_,_),
-    catch(existing_file_p(Name, Options, Rest), E, fail).
+    catch(existing_file_p(Name, FileOptions, Rest), E, fail).
 
-existing_file_p(Name, Options, Rest) :-
-    nb_current(pldoc_file, RelativeTo),
-    RelativeTo \== [],
+existing_file_p(Name, FileOptions, Rest) :-
+    (   nb_current(pldoc_file, RelativeTo),
+        RelativeTo \== []
+    ->  Extra = [relative_to(RelativeTo)|Extra1]
+    ;   Extra = Extra1
+    ),
     (   compound(Name)
-    ->  Extra = [file_type(prolog)]
-    ;   Extra = []
+    ->  Extra1 = [file_type(prolog)]
+    ;   Extra1 = []
     ),
     absolute_file_name(Name, Path,
-                       [ relative_to(RelativeTo),
-                         access(read),
+                       [ access(read),
                          file_errors(fail)
                        | Extra
                        ]),
-    Options = [ absolute_path(Path) | Rest ].
+    FileOptions = [ absolute_path(Path) | Rest ].
 
 %!  arity(-Arity:int)// is semidet.
 %
